"use client";

import React, { useEffect, useState } from "react";
import { ScrollArea } from "@/components/ui/scroll-area";
import {
  ResizableHandle,
  ResizablePanel,
  ResizablePanelGroup,
} from "@/components/ui/resizable";
import markdownStyles from "@/styles/markdown-styles.module.css";
import { Button, buttonVariants } from "@/components/ui/button";
import Link from "next/link";
import { cn } from "@/lib/utils";
import EditorWindow from "@/components/task/editor-window";
import Image from "next/image";
import ThemeToggle from "./layout/ThemeToggle/theme-toggle";
import { useQuizStore } from "@/lib/quiz-store";
import { useShallow } from "zustand/react/shallow";

interface LangTutorial {
  id: number;
  language: string;
  tutorial: {
    id: number;
    content: string;
    code: string;
  }[];
}

interface Tutorial {
  id: number;
  content: string;
  code: string;
}

const TaskPage = ({
  params,
  langTutorial,
  tutorial,
  tutorialIndex,
  content,
}: {
  params: { taskId: string[] };
  langTutorial: LangTutorial;
  tutorial: Tutorial;
  tutorialIndex: number;
  content: string;
}) => {
  const [activeMobileView, setActiveMobileView] = useState<
    "tutorial" | "code" | "output"
  >("tutorial");

  const { isTaskPageLoading, updateTaskPageLoading } = useQuizStore(
    useShallow((store) => ({
      isTaskPageLoading: store.isTaskPageLoading,
      updateTaskPageLoading: store.updateTaskPageLoading,
    })),
  );

  useEffect(() => {
    if (typeof window !== "undefined") {
      // console.log(isTaskPageLoading);
      updateTaskPageLoading(false);
    }
  }, []);

  return (
    <>
      <div className="md:hidden">
        <div
          className={cn(
            "flex items-center justify-center gap-4 fixed w-screen bottom-0 left-0 z-50 px-4 py-3 bg-background border-t transition-transform",
            isTaskPageLoading ? "translate-y-full" : "translate-y-0",
          )}
        >
          <Button
            variant={activeMobileView === "tutorial" ? "default" : "outline"}
            onClick={() => setActiveMobileView("tutorial")}
          >
            Tutorial
          </Button>
          <Button
            variant={activeMobileView === "code" ? "default" : "outline"}
            onClick={() => setActiveMobileView("code")}
          >
            Code
          </Button>
          <Button
            variant={activeMobileView === "output" ? "default" : "outline"}
            onClick={() => setActiveMobileView("output")}
          >
            Output
          </Button>
        </div>
      </div>
      <div className="md:hidden w-full h-full">
        <div
          className={cn(
            "fixed left-0 top-0 h-[calc(100vh-59px)] w-screen transition-transform translate-x-0",
            activeMobileView === "tutorial"
              ? "translate-x-0"
              : activeMobileView === "code"
              ? "-translate-x-full"
              : activeMobileView === "output"
              ? "-translate-x-[200%]"
              : "",
          )}
        >
          <div className="border-b px-4 bg-background">
            <nav className="flex justify-between items-center max-w-[1440px] mx-auto py-2">
              <Link href="/dashboard" className="inline-block w-fit">
                <Image
                  src="/Skilld AI Logos/Skilld Logo-colour on black.png"
                  alt="Skilld Logo"
                  width={100}
                  height={30}
                  className="hidden dark:block h-[25px] w-[80px] md:w-[100px] md:h-[30px] object-contain"
                />
                <Image
                  src="/Skilld AI Logos/Skilld Logo-colour.png"
                  alt="Skilld Logo"
                  width={100}
                  height={30}
                  className="dark:hidden h-[25px] w-[80px] md:w-[100px] md:h-[30px] object-contain"
                />
              </Link>

              <div className="flex items-center gap-8">
                <Link href="/dashboard">Dashboard</Link>
                <ThemeToggle />
              </div>
            </nav>
          </div>

          <ScrollArea className="h-[calc(100vh-calc(53px+59px))]">
            <div className="p-4 flex flex-col gap-4 h-full mb-16">
              <div
                className={cn("grow", markdownStyles["markdown"])}
                dangerouslySetInnerHTML={{ __html: content }}
              ></div>
              <div className="flex justify-between">
                <div>
                  {langTutorial?.tutorial[tutorialIndex - 1] && (
                    <Link
                      onClick={() => updateTaskPageLoading(true)}
                      href={`/dashboard/task/${params.taskId[0]}/${
                        Number(params.taskId[1]) - 1
                      }`}
                      className={buttonVariants({
                        variant: "outline",
                      })}
                    >
                      Prev
                    </Link>
                  )}
                </div>

                <div>
                  {langTutorial?.tutorial[tutorialIndex + 1] && (
                    <Link
                      onClick={() => updateTaskPageLoading(true)}
                      href={`/dashboard/task/${params.taskId[0]}/${
                        Number(params.taskId[1]) + 1
                      }`}
                      className={buttonVariants({
                        variant: "outline",
                      })}
                    >
                      Next
                    </Link>
                  )}
                </div>
              </div>
            </div>
          </ScrollArea>
        </div>

        <EditorWindow
          activeMobileView={activeMobileView}
          language={params.taskId[0]}
          step={Number(params.taskId[1]) || 1}
        />
      </div>

      <div className="hidden md:block w-full h-full">
        <ResizablePanelGroup
          direction="horizontal"
          className="max-w-[1440px] mx-auto h-full"
        >
          <ResizablePanel defaultSize={50}>
            <ScrollArea className="h-[calc(100vh-55px)]">
              <div className="px-4 py-6 flex flex-col gap-4 h-full mb-6">
                <div
                  className={cn("grow", markdownStyles["markdown"])}
                  dangerouslySetInnerHTML={{ __html: content }}
                ></div>

                <div className="flex justify-between">
                  <div>
                    {langTutorial?.tutorial[tutorialIndex - 1] && (
                      <Link
                        href={`/dashboard/task/${params.taskId[0]}/${
                          Number(params.taskId[1]) - 1
                        }`}
                        className={buttonVariants({
                          variant: "outline",
                        })}
                      >
                        Prev
                      </Link>
                    )}
                  </div>

                  <div>
                    {langTutorial?.tutorial[tutorialIndex + 1] && (
                      <Link
                        href={`/dashboard/task/${params.taskId[0]}/${
                          Number(params.taskId[1]) + 1
                        }`}
                        className={buttonVariants({ variant: "outline" })}
                      >
                        Next
                      </Link>
                    )}
                  </div>
                </div>
              </div>
            </ScrollArea>
          </ResizablePanel>
          <ResizableHandle withHandle />
          <ResizablePanel defaultSize={50}>
            <EditorWindow
              activeMobileView={activeMobileView}
              language={params.taskId[0]}
              step={Number(params.taskId[1]) || 1}
            />
          </ResizablePanel>
        </ResizablePanelGroup>
      </div>
    </>
  );
};

export default TaskPage;
